@@ -48,6 +48,7 @@ from uuid import UUID
 
 import attr
 
+from hypothesis._settings import note_deprecation
 from hypothesis.control import cleanup, note, reject
 from hypothesis.errors import InvalidArgument, ResolutionFailed
 from hypothesis.internal.cache import LRUReusedCache
@@ -119,7 +120,6 @@ from hypothesis.strategies._internal.strings import (
     OneCharStringStrategy,
     StringStrategy,
 )
-from hypothesis.types import RandomWithSeed
 from hypothesis.utils.conventions import InferType, infer, not_set
 
 K = TypeVar("K")
@@ -191,12 +191,18 @@ def cacheable(fn: T) -> T:
     return cached_strategy
 
 
-def base_defines_strategy(force_reusable: bool) -> Callable[[T], T]:
+def base_defines_strategy(
+    force_reusable: bool, *, try_non_lazy: bool = False
+) -> Callable[[T], T]:
     """Returns a decorator for strategy functions.
 
     If force_reusable is True, the generated values are assumed to be
     reusable, i.e. immutable and safe to cache, across multiple test
     invocations.
+
+    If try_non_lazy is True, attempt to execute the strategy definition
+    function immediately, so that a LazyStrategy is only returned if this
+    raises an exception.
     """
 
     def decorator(strategy_definition):
@@ -206,6 +212,16 @@ def base_defines_strategy(force_reusable: bool) -> Callable[[T], T]:
 
         @proxies(strategy_definition)
         def accept(*args, **kwargs):
+            if try_non_lazy:
+                # Why not try this unconditionally?  Because we'd end up with very
+                # deep nesting of recursive strategies - better to be lazy unless we
+                # *know* that eager evaluation is the right choice.
+                try:
+                    return strategy_definition(*args, **kwargs)
+                except Exception:
+                    # If invoking the strategy definition raises an exception,
+                    # wrap that up in a LazyStrategy so it happens again later.
+                    pass
             result = LazyStrategy(strategy_definition, args, kwargs)
             if force_reusable:
                 result.force_has_reusable_values = True
@@ -220,6 +236,7 @@ def base_defines_strategy(force_reusable: bool) -> Callable[[T], T]:
 
 defines_strategy = base_defines_strategy(False)
 defines_strategy_with_reusable_values = base_defines_strategy(True)
+defines_strategy_without_laziness = base_defines_strategy(False, try_non_lazy=True)
 
 
 class Nothing(SearchStrategy):
@@ -644,7 +661,7 @@ def sampled_from(elements: Type[enum.Enum]) -> SearchStrategy[Any]:
     pass  # pragma: no cover
 
 
-@defines_strategy  # noqa: F811
+@defines_strategy_without_laziness  # noqa: F811
 def sampled_from(elements):
     """Returns a strategy which generates any value present in ``elements``.
 
@@ -665,13 +682,19 @@ def sampled_from(elements):
         raise InvalidArgument("Cannot sample from a length-zero sequence.")
     if len(values) == 1:
         return just(values[0])
+    if isinstance(elements, type) and issubclass(elements, enum.Enum):
+        repr_ = "sampled_from(%s.%s)" % (elements.__module__, elements.__name__)
+    else:
+        repr_ = "sampled_from(%r)" % (elements,)
     if hasattr(enum, "Flag") and isclass(elements) and issubclass(elements, enum.Flag):
         # Combinations of enum.Flag members are also members.  We generate
         # these dynamically, because static allocation takes O(2^n) memory.
-        return sets(sampled_from(values), min_size=1).map(
+        # LazyStrategy is used for the ease of force_repr.
+        inner = sets(sampled_from(list(values)), min_size=1).map(
             lambda s: reduce(operator.or_, s)
         )
-    return SampledFromStrategy(elements)
+        return LazyStrategy(lambda: inner, args=[], kwargs={}, force_repr=repr_)
+    return SampledFromStrategy(values, repr_)
 
 
 @cacheable
@@ -1139,13 +1162,38 @@ def binary(*, min_size: int = 0, max_size: int = None) -> SearchStrategy[bytes]:
 
 @cacheable
 @defines_strategy
-def randoms() -> SearchStrategy[random.Random]:
-    """Generates instances of ``random.Random``, tweaked to show the seed value
-    in the repr for reproducibility.
+def randoms(
+    *, note_method_calls: bool = False, use_true_random: bool = None
+) -> SearchStrategy[random.Random]:
+    """Generates instances of ``random.Random``. The generated Random instances
+    are of a special HypothesisRandom subclass.
 
-    Examples from this strategy shrink to seeds closer to zero.
+    - If ``note_method_calls`` is set to ``True``, Hypothesis will print the
+      randomly drawn values in any falsifying test case. This can be helpful
+      for debugging the behaviour of randomized algorithms.
+    - If ``use_true_random`` is set to ``True`` then values will be drawn from
+      their usual distribution, otherwise they will actually be Hypothesis
+      generated values (and will be shrunk accordingly for any failing test
+      case). Setting ``use_true_random=False`` will tend to expose bugs that
+      would occur with very low probability when it is set to True, and this
+      flag should only be set to True when your code relies on the distribution
+      of values for correctness.
     """
-    return integers().map(RandomWithSeed)
+    if use_true_random is None:
+        note_deprecation(
+            """Defaulting to old behaviour of use_true_random=True. If you want
+            to keep that behaviour, set use_true_random=True explicitly. If you
+            want the new behaviour (which will become the default in future),
+            set use_true_random=False.""",
+            since="2020-06-30",
+        )
+        use_true_random = True
+
+    from hypothesis.strategies._internal.random import RandomStrategy
+
+    return RandomStrategy(
+        use_true_random=use_true_random, note_method_calls=note_method_calls
+    )
 
 
 class RandomSeeder:
@@ -1912,9 +1960,9 @@ def uuids(*, version: int = None) -> SearchStrategy[UUID]:
             )
             % (version,)
         )
-    return shared(randoms(), key="hypothesis.strategies.uuids.generator").map(
-        lambda r: UUID(version=version, int=r.getrandbits(128))
-    )
+    return shared(
+        randoms(use_true_random=True), key="hypothesis.strategies.uuids.generator"
+    ).map(lambda r: UUID(version=version, int=r.getrandbits(128)))
 
 
 class RunnerStrategy(SearchStrategy):
@@ -2148,15 +2196,16 @@ def functions(
 def slices(draw: Any, size: int) -> slice:
     """Generates slices that will select indices up to the supplied size
 
-    Generated slices will have start and stop indices that range from 0 to size - 1
+    Generated slices will have start and stop indices that range from -size to size - 1
     and will step in the appropriate direction. Slices should only produce an empty selection
     if the start and end are the same.
 
     Examples from this strategy shrink toward 0 and smaller values
     """
-    check_valid_integer(size, "size")
-    if size is None or size < 1:
-        raise InvalidArgument("size=%r must be at least one" % size)
+    check_valid_size(size, "size")
+    if size == 0:
+        step = draw(none() | integers().filter(bool))
+        return slice(None, None, step)
 
     min_start = min_stop = 0
     max_start = max_stop = size
@@ -2180,4 +2229,9 @@ def slices(draw: Any, size: int) -> slice:
     if (stop or 0) < (start or 0):
         step *= -1
 
+    if draw(booleans()) and start is not None:
+        start -= size
+    if draw(booleans()) and stop is not None:
+        stop -= size
+
     return slice(start, stop, step)
